#include "def_vst.h"
#define  MAX_N 200

MAIN
{ 
    int      i,
             nf;                     /* # floats                    */
    long     vec_cnt;
    float    ufv[MAX_N],             /* Eingabevektor               */
             mean[MAX_N],            /* Mittelwert                  */
             sigma[MAX_N];           /* Standardabweichung          */

    bool     mask[MAX_N];
    char     *mask_string,
             line[81];

    FILE     *statistik_fd;
    char     *statistik_fn;



    static char *synopsis = 
"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"
"                                     V1.1,  27.04.1993   V.Strom \n"
"                                                                 \n"
" usage:  ufv_norm  stat_file  n  [-m mask]   < ufv   > ufv_norm  \n"
"                                                                 \n"
" < ufv       unlabeled feature vectors     (float[])             \n"
" n           vector length (# of floats)   (int)                 \n"
" stat_file   ASCII file containing the                           \n"
"             means and stddevs (n lines)   (string)              \n"
" mask        `[01]*'                       (string, len=n,       \n"
"                                            default = `111...1') \n"
" > ufv_norm  unlabeled feature vectors     (float[])             \n"
"                                                                 \n"
" Normalise feature vectors such that the mean of each component  \n"
" will be 0 and the standard deviation 1.  Get the stat_file by   \n"
"                                                                 \n"
"    ufv_statistik  ms  <ufv  >stat_file                          \n"
"                                                                 \n"
" The mask allows for excluding certain components from           \n"
" normalising.                                                    \n"
"                                                                 \n"
"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n";
    ARGS(2,"","m");
    
    SARG(1, statistik_fn);
    IARG(2,nf);
    if(nf >= MAX_N)
        ERROR1("vector size exceeded %d", MAX_N);

    SOPT('m',mask_string,NULL);

    /*------ mask_string --------------------------------------*/
    if(mask_string)
    {    if(strlen(mask_string)==nf)
         {   for(i=0; i<nf; i++)
                 if(mask_string[i]=='1')
                     mask[i] = TRUE;
                 else if(mask_string[i]=='0')
                     mask[i] = FALSE;
                 else
                     ERROR1("illegal mask \"%s\"",mask_string);
         }
         else
             ERROR1("illegal mask \"%s\"",mask_string);
    }
    else
    {   for(i=0; i<nf; i++)
            mask[i] = TRUE;
    }

    /*------ statistik_file ---------------------------------------*/
    FOPEN(statistik_fd, statistik_fn, "r");
    DMESSAGE0("index\tmean\tsigma");
    for(i=0; i<nf; i++)
    {   if(fgets(line, 80, statistik_fd)==NULL)
            ERROR2("file \"%s\": no line %d", statistik_fn, i+1);
        if(sscanf(line, "%g %g", &mean[i], &sigma[i]) != 2)
            ERROR2("file \"%s\": not 2 floats in line %d", statistik_fn, i+1);
        DMESSAGE3("%2d\t%g\t%g", i, mean[i], sigma[i]);
        
        if(sigma[i] == 0)
            ERROR1("sigma[%d] == 0", i);
    }

    /*------------ Hauptschleife: --------------------------*/
    while(FREAD(ufv, float, EXACTLY, nf, stdin))
    {   vec_cnt++;
        for(i=0; i<nf; i++)
        {   if(mask[i])
            {   ufv[i] -= mean[i];
                ufv[i] /= sigma[i];
            }
        }
        FWRITE(ufv, float, nf, stdout);
    }
    /*------------------------------------------------------*/
    
    return 0;
}   
